/**
 * @file registers the simpleBox toolbar button and binds functionality to it.
 */

import {Plugin} from 'ckeditor5/src/core';
import {ButtonView, clickOutsideHandler, ContextualBalloon} from 'ckeditor5/src/ui';
import SpanManagerView from './spanmanagerview';
import { Element } from 'ckeditor5/src/engine';
//import  getClosestSelectedSpanElement from "./utils";
//import  getRangeText  from "./utils";
import {getClosestSelectedSpanElement, findLinkElementAncestor, getRangeText, getRangesOfBlockGroups} from "./utils";


export default class SpanManagerUI extends Plugin {
  static get requires() {
    return [ContextualBalloon];
  }

  init() { 
    const editor = this.editor;

    // Create the balloon and the form view.
    this._balloon = this.editor.plugins.get(ContextualBalloon);
    this.formView = this._createFormView();

    // Register the button in the editor's UI component factory.
    editor.ui.componentFactory.add('spanManager', () => { 
      const button = new ButtonView();
      const command = editor.commands.get( 'addSpan' );

      button.set({
        label: 'Span',
        tooltip: true,
        withText: true,
        isToggleable: true
      });
      //button.bind( 'isEnabled' ).to( command, 'isEnabled', function(isEnabled){ return !isEnabled; console.log('isEnabled', isEnabled); } );
      button.bind( 'isOn' ).to( command, 'value', function(value){ return value;  } );

      this.listenTo(button, 'execute', () => {
        this._showUI();
        editor.editing.view.focus();
      });

      return button;
    });
  }

  _createFormView() {
    const editor = this.editor;
    const formView = new SpanManagerView(editor.locale);
    const command = editor.commands.get( 'addSpan' );

    // Execute the command after clicking the "Save" button.
    this.listenTo(formView, 'submit', () => {      
      const value = {
        content:   formView.textInputView.fieldView.element.value,
        title:     formView.titleInputView.fieldView.element.value,
        id:        formView.idInputView.fieldView.element.value,
        langcode:  formView.langcodeInputView.fieldView.element.value,
        className: formView.classInputView.fieldView.element.value,
        rawStyles: formView.styleInputView.fieldView.element.value,
      };
      this.editor.execute('addSpan', value);

      // Hide the form view after submit.
      this._hideUI();
    });

    // Hide the form view after clicking the "Cancel" button.
    this.listenTo(formView, 'cancel', () => { 
      this._hideUI();
    });

    // Hide the form view after clicking the "Cancel" button.
    this.listenTo(formView, 'remove', () => {
      const value = {
        content:   formView.textInputView.fieldView.element.value,
        title:     formView.titleInputView.fieldView.element.value,
        id:        formView.idInputView.fieldView.element.value,
        langcode:  formView.langcodeInputView.fieldView.element.value,
        className: formView.classInputView.fieldView.element.value,
        rawStyles: formView.styleInputView.fieldView.element.value,
      };
      this.editor.execute('removeSpan', value);      
      this._hideUI();
    });

    // Hide the form view when clicking outside the balloon.
    clickOutsideHandler({
      emitter: formView,
      activator: () => this._balloon.visibleView === formView,
      contextElements: [this._balloon.view.element],
      callback: () => this._hideUI()
    });

    return formView;
  }

  _getBalloonPositionData() {
    const view = this.editor.editing.view;
    const viewDocument = view.document;
    let target = null;

    // Set a target position by converting view selection range to DOM.
    target = () => view.domConverter.viewRangeToDom(viewDocument.selection.getFirstRange());

    return {
      target
    };
  }

/*
  _getSelectedLinkElement() {
		const view = this.editor.editing.view;
		const selection = view.document.selection;
		const selectedElement = selection.getSelectedElement();

		// The selection is collapsed or some widget is selected (especially inline widget).
		if ( selection.isCollapsed || selectedElement && isWidget( selectedElement ) ) { console.log('iiiiiii')
			return findLinkElementAncestor( selection.getFirstPosition() );
    } 
    else {
			// The range for fully selected link is usually anchored in adjacent text nodes.
			// Trim it to get closer to the actual link element.
			const range = selection.getFirstRange().getTrimmed();
			const startLink = findLinkElementAncestor( range.start );
			const endLink = findLinkElementAncestor( range.end );

			if ( !startLink || startLink != endLink ) {
				return null;
			}

			// Check if the link element is fully selected.
			if ( view.createRangeIn( startLink ).getTrimmed().isEqual( range ) ) {
				return startLink;
			} else {
				return null;
			}
		}
	}
*/
  _showUI() {
    const selection = this.editor.model.document.selection;
		const linkCommand = this.editor.commands.get( 'addSpan' );

    const existingSpan = getClosestSelectedSpanElement(selection);

    this._balloon.add({
      view: this.formView,
      position: this._getBalloonPositionData()
    });

    
    // Disable the input when the selection is not collapsed.
    this.formView.textInputView.isEnabled = selection.getFirstRange().isCollapsed;


    var defaultId = '';
    var defaultClass = '';
    var defaultLangcode = '';
    var defaultStyle = '';
    var defaultTitle = '';
    if(existingSpan){
      
      const data = existingSpan.getChild(0).data;
      this.formView.textInputView.fieldView.value = data;
      this.formView.textInputView.element.classList.add("ui-snippet-hidden");
      this.formView.textInputView.fieldView.element.classList.add("ui-snippet-hidden");
      //this.formView.textInputView.fieldView.classList.add("ui-snippet-hidden");
      this.formView.textInputView.set('class', "ui-snippet-hidden");
      //console.log('ici', this.formView.textInputView.fieldView.get('class'))
      
      defaultId = existingSpan.getAttribute('id');
      defaultClass = existingSpan.getAttribute('class');
      defaultLangcode = existingSpan.getAttribute('lang');
      defaultStyle = existingSpan.getAttribute('style');
      defaultTitle = existingSpan.getAttribute('title');
    }
    else {
      this.formView.textInputView.element.classList.remove("ui-snippet-hidden");
      this.formView.textInputView.fieldView.element.classList.remove("ui-snippet-hidden");
      this.formView.textInputView.set('class', "");
      this.formView.textInputView.fieldView.value = '';
    }
    this.formView.idInputView.fieldView.value = defaultId;
    this.formView.classInputView.fieldView.value = defaultClass;
    this.formView.langcodeInputView.fieldView.value = defaultLangcode;
    this.formView.styleInputView.fieldView.value = defaultStyle;
    this.formView.titleInputView.fieldView.value = defaultTitle;
    
    this.formView.focus(); 
  }

  _hideUI() {
    // Clear the input field values and reset the form.
    this.formView.textInputView.fieldView.value = '';
    this.formView.idInputView.fieldView.value = '';
    this.formView.classInputView.fieldView.value = '';
    this.formView.langcodeInputView.fieldView.value = '';
    this.formView.styleInputView.fieldView.value = '';
    this.formView.titleInputView.fieldView.value = '';
    this.formView.element.reset();

    this._balloon.remove(this.formView);

    // Focus the editing view after inserting the span so the user can start typing the content
    // right away and keep the editor focused.
    this.editor.editing.view.focus();
  }
}

